@@ -18,6 +18,7 @@ from typing import Optional
 from typing import NamedTuple
 from cftool.misc import safe_execute
 from cftool.misc import shallow_copy_dict
+from cftool.misc import context_error_handler
 from cftool.array import arr_type
 from cftool.array import save_images
 from cftool.types import tensor_dict_type
@@ -229,6 +230,7 @@ class DiffusionAPI(APIMixin):
         registered_custom = False
         if self.cond_model is not None:
             if isinstance(self.cond_model, CLIPTextConditionModel):
+                self.cond_model.clip_skip = kwargs.get("clip_skip", 0)
                 custom_embeddings = kwargs.get("custom_embeddings")
                 if custom_embeddings is not None:
                     registered_custom = True
@@ -362,6 +364,7 @@ class DiffusionAPI(APIMixin):
         if callback is not None:
             concat = callback(concat)
         if original_size is not None:
+            original_size = (max(original_size[0], 1), max(original_size[1], 1))
             with torch.no_grad():
                 concat = F.interpolate(
                     concat,
@@ -483,10 +486,35 @@ class DiffusionAPI(APIMixin):
         max_wh: int = 512,
         num_steps: Optional[int] = None,
         clip_output: bool = True,
+        use_raw_inpainting: bool = False,
+        raw_inpainting_fidelity: float = 0.2,
         callback: Optional[Callable[[Tensor], Tensor]] = None,
         verbose: bool = True,
         **kwargs: Any,
     ) -> Tensor:
+        if use_raw_inpainting:
+            image_res = read_image(image, max_wh, anchor=anchor)
+            mask = read_image(mask, max_wh, anchor=anchor, to_mask=True).image
+            z = z_ref = self._get_z(image_res.image)
+            z_ref_mask = F.interpolate(
+                torch.from_numpy(mask).to(z_ref),
+                z_ref.shape[-2:],
+                mode="bicubic",
+            )
+            return self._img2img(
+                z,
+                export_path,
+                z_ref=z_ref,
+                z_ref_mask=z_ref_mask,
+                fidelity=raw_inpainting_fidelity,
+                original_size=image_res.original_size,
+                alpha=None,
+                cond=[txt],
+                num_steps=num_steps,
+                clip_output=clip_output,
+                verbose=verbose,
+                **kwargs,
+            )
         txt_list, num_samples = self._txt_cond(txt, 1)
         res = self._get_masked_cond(
             image,
@@ -734,6 +762,26 @@ class DiffusionAPI(APIMixin):
             **kwargs,
         )
 
+    def load_context(self) -> context_error_handler:
+        class _(context_error_handler):
+            def __init__(self, api: DiffusionAPI):
+                self.api = api
+                self.m_cond = api.m.condition_model
+                api.m.condition_model = api.cond_model
+
+            def __enter__(self) -> nn.Module:
+                class Wrapper(nn.Module):
+                    def __init__(self, m: nn.Module) -> None:
+                        super().__init__()
+                        self.core = m
+
+                return Wrapper(self.api.m)
+
+            def _normal_exit(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
+                self.api.m.condition_model = self.m_cond
+
+        return _(self)
+
     @classmethod
     def from_sd(
         cls,
