@@ -3,19 +3,21 @@ import torch
 
 import numpy as np
 
+from cftool.array import to_numpy
+from cftool.array import to_torch
+from cftool.array import to_device
+from cftool.array import save_images
+
 from .general import ImageCallback
-from ...toolkit import to_numpy
-from ...toolkit import to_torch
-from ...toolkit import to_device
-from ...toolkit import save_images
 from ...toolkit import eval_context
-from ....trainer import Trainer
-from ....trainer import TrainerCallback
+from ....protocol import _forward
+from ....protocol import ITrainer
+from ....protocol import TrainerCallback
 from ....constants import INPUT_KEY
 from ....constants import LABEL_KEY
+from ....constants import STYLE_KEY
 from ....constants import PREDICTIONS_KEY
-from ....models.cv.protocol import GeneratorMixin
-from ....models.cv.stylizer.constants import STYLE_KEY
+from ....models.protocols.cv import GeneratorMixin
 from ....models.cv.segmentor.constants import LV1_ALPHA_KEY
 
 try:
@@ -24,6 +26,10 @@ except:
     min_max_normalize = None
 
 
+@TrainerCallback.register("ldm")
+@TrainerCallback.register("ddpm")
+@TrainerCallback.register("ae_kl")
+@TrainerCallback.register("ae_vq")
 @TrainerCallback.register("gan")
 @TrainerCallback.register("vae")
 @TrainerCallback.register("vae2d")
@@ -34,17 +40,17 @@ class GeneratorCallback(ImageCallback):
         super().__init__(num_keep)
         self.num_interpolations = num_interpolations
 
-    def log_artifacts(self, trainer: Trainer) -> None:
+    def log_artifacts(self, trainer: ITrainer) -> None:
         if not self.is_rank_0:
             return None
         batch = next(iter(trainer.validation_loader))
         batch = to_device(batch, trainer.device)
         original = batch[INPUT_KEY]
-        model = trainer.model
+        model = trainer.model.core
         if not isinstance(model, GeneratorMixin):
             msg = "`GeneratorCallback` is only compatible with `GeneratorMixin`"
             raise ValueError(msg)
-        is_conditional = model.is_conditional
+        is_conditional = getattr(model, "is_conditional", False)
         labels = None if not is_conditional else batch[LABEL_KEY]
         image_folder = self._prepare_folder(trainer)
         # original
@@ -63,12 +69,12 @@ class GeneratorCallback(ImageCallback):
             interpolations = model.interpolate(self.num_interpolations)
         save_images(interpolations, os.path.join(image_folder, "interpolations.png"))
         # conditional sampling
-        if model.num_classes is None:
+        if getattr(model, "num_classes", None) is None:
             return None
         cond_folder = os.path.join(image_folder, "conditional")
         os.makedirs(cond_folder, exist_ok=True)
         with eval_context(model):
-            for i in range(model.num_classes):
+            for i in range(model.num_classes or 0):
                 sampled = model.sample(len(original), class_idx=i)
                 interpolations = model.interpolate(len(original), class_idx=i)
                 save_images(sampled, os.path.join(cond_folder, f"sampled_{i}.png"))
@@ -80,11 +86,11 @@ class GeneratorCallback(ImageCallback):
 @TrainerCallback.register("siren_vae")
 @TrainerCallback.register("sized_generator")
 class SizedGeneratorCallback(GeneratorCallback):
-    def log_artifacts(self, trainer: Trainer) -> None:
+    def log_artifacts(self, trainer: ITrainer) -> None:
         if not self.is_rank_0:
             return None
         super().log_artifacts(trainer)
-        model = trainer.model
+        model = trainer.model.core
         image_folder = self._prepare_folder(trainer, check_num_keep=False)
         sample_method = getattr(model, "sample", None)
         if sample_method is None:
@@ -106,13 +112,14 @@ class SizedGeneratorCallback(GeneratorCallback):
 @ImageCallback.register("u2net")
 @ImageCallback.register("cascade_u2net")
 class AlphaSegmentationCallback(ImageCallback):
-    def log_artifacts(self, trainer: Trainer) -> None:
+    def log_artifacts(self, trainer: ITrainer) -> None:
         if not self.is_rank_0:
             return None
         batch = next(iter(trainer.validation_loader))
         batch = to_device(batch, trainer.device)
-        with eval_context(trainer.model):
-            results = trainer.model(0, batch)
+        model = trainer.model.core
+        with eval_context(model):
+            results = _forward(model, 0, batch, INPUT_KEY)
         original = batch[INPUT_KEY]
         label = batch[LABEL_KEY].float()
         logits = results[PREDICTIONS_KEY]
@@ -147,14 +154,14 @@ class AlphaSegmentationCallback(ImageCallback):
 @TrainerCallback.register("adain")
 @TrainerCallback.register("style_transfer")
 class StyleTransferCallback(ImageCallback):
-    def log_artifacts(self, trainer: Trainer) -> None:
+    def log_artifacts(self, trainer: ITrainer) -> None:
         if not self.is_rank_0:
             return None
         batch = next(iter(trainer.validation_loader))
         batch = to_device(batch, trainer.device)
         content = batch[INPUT_KEY]
         style = batch[STYLE_KEY]
-        model = trainer.model
+        model = trainer.model.core
         image_folder = self._prepare_folder(trainer)
         # inputs
         save_images(content, os.path.join(image_folder, "original.png"))
