@@ -27,7 +27,7 @@ class VQVAE(nn.Module, WithDeviceMixin, ApplyTanhMixin):
         out_channels: Optional[int] = None,
         num_downsample: Optional[int] = None,
         min_size: int = 8,
-        target_downsample: int = None,
+        target_downsample: Optional[int] = None,
         *,
         encoder: str = "vanilla",
         decoder: str = "vanilla",
