@@ -1,15 +1,15 @@
 from typing import Optional
 
-from .protocol import MixedStackedModel
-from .protocol import register_ml_module
+from ..register import register_ml_module
+from ..protocols.ml import MixedStackedModel
 
 
 @register_ml_module("fnet")
 class FNet(MixedStackedModel):
     def __init__(
         self,
-        in_dim: int,
-        out_dim: int,
+        input_dim: int,
+        output_dim: int,
         num_history: int,
         latent_dim: int = 256,
         *,
@@ -18,8 +18,8 @@ class FNet(MixedStackedModel):
         norm_type: Optional[str] = "batch_norm",
     ):
         super().__init__(
-            in_dim,
-            out_dim,
+            input_dim,
+            output_dim,
             num_history,
             latent_dim,
             token_mixing_type="fourier",
