@@ -7,14 +7,14 @@ from typing import Callable
 from typing import Optional
 from cftool.misc import check_requires
 from cftool.misc import shallow_copy_dict
+from cftool.types import np_dict_type
+from cftool.types import tensor_dict_type
 
-from ...bases import ModelProtocol
+from ...bases import IDLModel
+from ....data import TensorDictData
 from ....types import texts_type
-from ....types import np_dict_type
-from ....types import tensor_dict_type
 from ....protocol import TrainerState
-from ....protocol import InferenceProtocol
-from ....data.interface import TensorDictData
+from ....protocol import IInference
 
 try:
     from transformers import AutoModel
@@ -23,8 +23,8 @@ except:
     AutoModel = AutoTokenizer = None
 
 
-@ModelProtocol.register("hugging_face")
-class HuggingFaceModel(ModelProtocol):
+@IDLModel.register("hugging_face")
+class HuggingFaceModel(IDLModel):
     tokenizer_base: Type = AutoTokenizer
     model_base: Type = AutoModel
     forward_fn_name: str = "forward"
@@ -60,10 +60,8 @@ class HuggingFaceModel(ModelProtocol):
         **kwargs: Any,
     ) -> np_dict_type:
         x = self.tokenizer(texts, padding=True, return_tensors="pt")
-        data = TensorDictData(x)
-        data.prepare(None)
-        loader = data.initialize()[0]
-        inference = InferenceProtocol(model=self)
+        loader = TensorDictData(x).get_loaders()[0]
+        inference = IInference(model=self)
         outputs = inference.get_outputs(loader, use_tqdm=use_tqdm, **kwargs)
         return outputs.forward_results
 
@@ -82,7 +80,7 @@ class HuggingFaceModel(ModelProtocol):
         num_samples: Optional[int] = None,
         verbose: bool = True,
         **kwargs: Any,
-    ) -> "ModelProtocol":
+    ) -> "IDLModel":
         if max_length is None:
             input_sample = self.tokenizer(sample_text, return_tensors="pt")
         else:
